@@ -6,6 +6,7 @@ use Pcore::Handle::pgsql qw[:ALL];
 use Pcore::Util::Scalar qw[weaken looks_like_number is_plain_arrayref is_plain_coderef is_blessed_arrayref];
 use Pcore::Util::UUID qw[uuid_str];
 use Pcore::Util::Digest qw[md5_hex];
+use Pcore::Util::Data qw[from_json];
 use Pcore::AE::Handle;
 
 has handle     => ( is => 'ro', isa => InstanceOf ['Pcore::Handle::pgsql'], required => 1 );
@@ -429,6 +430,15 @@ sub _ON_DATA_ROW ( $self, $dataref ) {
                     $col = $col eq 'f' ? 0 : 1;
                 }
 
+                # TODO decode ARRAY
+                # elsif ( $type == $SQL_TEXTARRAY ) {
+                # }
+
+                # decode JSON
+                elsif ( $type == $SQL_JSON ) {
+                    $col = from_json $col;
+                }
+
                 # decode text value
                 else {
                     utf8::decode $col;
@@ -567,11 +577,9 @@ sub _execute ( $self, $query, $bind, $cb, %args ) {
     else {
 
         # convert "?" placeholders to postgres "$1" style
-        if ( index( $query, '?' ) != -1 ) {
-            my $i;
+        my $i;
 
-            $query =~ s/[?]/'$' . ++$i/smge;
-        }
+        $query =~ s/[?]/'$' . ++$i/smge;
 
         utf8::encode $query if utf8::is_utf8 $query;
     }
@@ -606,24 +614,32 @@ sub _execute ( $self, $query, $bind, $cb, %args ) {
 
             for my $param (@bind) {
                 if ( is_blessed_arrayref $param) {
-                    if ( $param->[1] == $SQL_BYTEA ) {
+                    if ( $param->[0] == $SQL_BYTEA ) {
                         $param_format_codes .= "\x00\x01";    # binary
 
-                        $param = $param->[0];
-                    }
-                    elsif ( $param->[1] == $SQL_BOOL ) {
-                        $param_format_codes .= "\x00\x00";    # text
-
-                        $param = $param->[0] ? '1' : '0';
+                        $param = $param->[1];
                     }
                     else {
                         $param_format_codes .= "\x00\x00";    # text
 
-                        $param = $param->[0];
+                        if ( $param->[0] == $SQL_BOOL ) {
+                            $param = $param->[1] ? '1' : '0';
+                        }
+                        elsif ( $param->[0] == $SQL_JSON ) {
+                            $param = $self->encode_json( $param->[1] )->$*;
+                        }
+                        else {
+                            $param = $param->[1];
+                        }
                     }
                 }
+                elsif ( is_plain_arrayref $param) {
+                    $param_format_codes .= "\x00\x00";    # text
+
+                    $param = $self->encode_array($param)->$*;
+                }
                 else {
-                    $param_format_codes .= "\x00\x00";        # text
+                    $param_format_codes .= "\x00\x00";    # text
                 }
 
                 if ( defined $param ) {
@@ -971,8 +987,16 @@ sub quote_id ( $self, $id ) {
     return $self->{handle}->quote_id($id);
 }
 
-sub quote ( $self, $var, $type = undef ) {
-    return $self->{handle}->quote( $var, $type );
+sub quote ( $self, $var ) {
+    return $self->{handle}->quote($var);
+}
+
+sub encode_array ( $self, $var ) {
+    return $self->{handle}->encode_array($var);
+}
+
+sub encode_json ( $self, $var ) {
+    return $self->{handle}->encode_json($var);
 }
 
 1;
@@ -982,20 +1006,20 @@ sub quote ( $self, $var, $type = undef ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 180                  | ControlStructures::ProhibitCascadingIfElse - Cascading if-elsif chain                                          |
+## |    3 | 181                  | ControlStructures::ProhibitCascadingIfElse - Cascading if-elsif chain                                          |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 525                  | Subroutines::ProhibitExcessComplexity - Subroutine "_execute" with high complexity score (27)                  |
+## |    3 | 535                  | Subroutines::ProhibitExcessComplexity - Subroutine "_execute" with high complexity score (29)                  |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 899                  | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
+## |    3 | 625, 915             | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 27, 150, 358, 516,   | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
-## |      | 584, 594, 610, 615,  |                                                                                                                |
-## |      | 620, 626, 635, 642,  |                                                                                                                |
-## |      | 646, 650, 654, 657   |                                                                                                                |
+## |    2 | 28, 151, 359, 526,   | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |      | 592, 602, 618, 623,  |                                                                                                                |
+## |      | 637, 642, 651, 658,  |                                                                                                                |
+## |      | 662, 666, 670, 673   |                                                                                                                |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 736, 899             | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
+## |    2 | 752, 915             | ControlStructures::ProhibitCStyleForLoops - C-style "for" loop used                                            |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 775                  | ControlStructures::ProhibitPostfixControls - Postfix control "for" used                                        |
+## |    2 | 791                  | ControlStructures::ProhibitPostfixControls - Postfix control "for" used                                        |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
